from typing import NewType, Dict, Any, List, cast, Optional

from druidapi.client import _common

DruidTaskId = NewType('DruidTaskId', str)
DruidSupervisorId = NewType('DruidSupervisorId', str)

DruidTaskSpecification = NewType('DruidTaskSpecification', Dict[str, Any])
DruidSupervisorSpecification = NewType('DruidSupervisorSpecification', Dict[str, Any])


class DoesNotExistException(Exception):
    pass


class TaskAlreadyExistsException(Exception):
    pass


class OverlordClient(_common.CommonClient):
    def supervisor_get_all(self) -> List[str]:
        return cast(List[str], self._get('/druid/indexer/v1/supervisor').json())

    def supervisor_get(self, supervisorid: DruidSupervisorId) -> DruidSupervisorSpecification:
        response = self._get(f'/druid/indexer/v1/supervisor/{supervisorid}/')
        if response.status_code == 200:
            return cast(DruidSupervisorSpecification, response.json())
        if response.status_code == 400:
            if response.text.startswith('Cannot find any supervisor with id'):
                raise DoesNotExistException(response.text)

        raise Exception(f'{response.status_code}: {response.text}')

    def supervisor_post(self, supervisor: DruidSupervisorSpecification) -> DruidSupervisorSpecification:
        response = self._post('/druid/indexer/v1/supervisor', json=supervisor)
        if response.status_code == 200:
            return cast(DruidSupervisorSpecification, response.json())

        raise Exception(f'{response.status_code}: {response.text}')

    def task_get_all(self,
                     datasource: Optional[str] = None,
                     state: Optional[str] = None,
                     type_: Optional[str] = None,
                     max_: int = 15) -> List[Dict[str, Any]]:
        params: Dict[str, str] = {'max': str(max_)}
        if datasource is not None:
            params['datasource'] = datasource
        if state is not None:
            params['state'] = state
        if type_ is not None:
            params['type'] = type_

        return cast(List[Dict[str, Any]], self._get(f'/druid/indexer/v1/tasks', params=params).json())

    def task_status_get(self, taskid: DruidTaskId) -> Dict[str, Any]:
        response = self._get(f'/druid/indexer/v1/task/{taskid}/status')
        if response.status_code == 200:
            return cast(Dict[str, Any], response.json())

        raise Exception(f'{response.status_code}: {response.text}')

    def task_log_get(self, taskid: DruidTaskId) -> str:
        return self._get(f'/druid/indexer/v1/task/{taskid}/log').content.decode('utf8')

    def task_post(self, task: DruidTaskSpecification) -> DruidTaskId:
        response = self._post('/druid/indexer/v1/task', json=task)
        if response.status_code == 200:
            return cast(DruidTaskId, response.json()['task'])
        if response.status_code == 400:
            content = response.json()
            if content['error'].find('already exists') >= 0:
                raise TaskAlreadyExistsException(content['error'])

        raise Exception(f'{response.status_code}: {response.text}')
