import logging
from datetime import datetime
from time import sleep
from typing import Optional, Iterable, Tuple

from druidapi import configuration
from druidapi.client.coordinator import CoordinatorClient
from druidapi.client.overlord import OverlordClient, DruidTaskSpecification, DruidTaskId


class DruidClient:
    overlord: OverlordClient
    coordinator: CoordinatorClient

    def __init__(self, config: configuration.DruidApiClientConfiguration):
        self.logger = logging.getLogger('druidclient')

        self.overlord = OverlordClient(config)
        self.coordinator = CoordinatorClient(config)

    def run_task(self, task: DruidTaskSpecification,
                 intervals: Optional[Iterable[Tuple[datetime, datetime]]] = None,
                 overwrite_name: Optional[DruidTaskId] = None) -> DruidTaskId:
        if intervals is not None:
            for interval in intervals:
                if not len(interval) == 2:
                    raise ValueError('Every interval must be a 2-element long tuple/list')
                if not isinstance(interval[0], datetime) or interval[0].tzinfo is None:
                    raise ValueError('Element 0 of an interval must be timezone aware datetime. Was:', interval[0])
                if not isinstance(interval[1], datetime) or interval[1].tzinfo is None:
                    raise ValueError('Element 1 of an interval must be timezone aware datetime. Was:', interval[1])

            druid_intervals = list([interval[0].isoformat() + '/' + interval[1].isoformat() for interval in intervals])
            task['spec']['dataSchema']['granularitySpec']['intervals'] = druid_intervals

        if overwrite_name is not None:
            task['id'] = overwrite_name

        taskid = self.overlord.task_post(task)
        self.logger.info(f'Created Task: {taskid}')

        return taskid

    def run_task_blocking(self, task: DruidTaskSpecification,
                          intervals: Optional[Iterable[Tuple[datetime, datetime]]] = None,
                          overwrite_name: Optional[DruidTaskId] = None) -> Tuple[str, Optional[str]]:
        taskid = self.run_task(task, intervals=intervals, overwrite_name=overwrite_name)

        while True:
            status = self.overlord.task_status_get(taskid)
            status_code = status['status']['statusCode']

            if status_code != 'RUNNING':
                break

            sleep(10)

        error_message = None
        if status_code == 'FAILED':
            error_message = status['status']['errorMsg']
        return status_code, error_message
